#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


## Default initialization for flow tests
#
# Adrien Crovato

from fwk.wutils import parseargs

## Initialize Picard solver
#
# Adrien Crovato
def initPicard(_solver):
    args = parseargs()
    _solver.nthreads = args.k
    _solver.relTol = 1e-6
    _solver.absTol = 1e-8
    _solver.maxIt = 25
    _solver.relax = 0.7

## Initialize Newton solver
#
# Adrien Crovato
def initNewton(_solver):
    args = parseargs()
    _solver.nthreads = args.k
    _solver.relTol = 1e-6
    _solver.absTol = 1e-8
    _solver.maxIt = 25
    _solver.lsTol = 1e-6
    _solver.maxLsIt = 10
    _solver.avThrsh = 1e-2

## Initialize mesh deformer
#
# Adrien Crovato
def initMeshDeformer(_mshDef):
    args = parseargs()
    _mshDef.nthreads = args.k

## Initialize viewer
#
# Adrien Crovato
def initViewer(problem):
    args = parseargs()
    if not args.nogui:
        from tbox.viewer import GUI
        GUI().open( problem.msh.name )
