/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WMEDIUM_H
#define WMEDIUM_H

#include "flow.h"
#include "wGroup.h"
#include "wFct0.h"
#include "wF0Ps.h"

namespace flow
{

/**
 * @brief Handle fluid medium
 * @authors Adrien Crovato
 */
class FLOW_API Medium : public tbox::Group
{
public:
    std::map<tbox::Node *, std::vector<tbox::Element *>> neMap; ///< map each elements connected to a node
    size_t nCnt;                                                ///< number of nodes

#ifndef SWIG
    tbox::Fct0 &rho;             ///< density
    tbox::Fct0 &mach;            ///< Mach number
    tbox::Fct0 &cP;              ///< pressure coefficient
    flow::F0PsPhiInf &phiInf; ///< freestream potential
#endif

    std::vector<std::pair<tbox::Element *, std::vector<tbox::Element *>>> adjMap; ///< map of adjacent elements

    Medium(std::shared_ptr<tbox::MshData> _msh, int no, tbox::Fct0 &_rho, tbox::Fct0 &_mach, tbox::Fct0 &_cP, flow::F0PsPhiInf &_phiInf);
    Medium(std::shared_ptr<tbox::MshData> _msh, std::string const &name, tbox::Fct0 &_rho, tbox::Fct0 &_mach, tbox::Fct0 &_cP, flow::F0PsPhiInf &_phiInf);
    virtual ~Medium() { std::cout << "~Medium()\n"; }

#ifndef SWIG
    void createMap();
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace flow

#endif //WMEDIUM_H
