/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WPICARD_H
#define WPICARD_H

#include "flow.h"
#include "wSolver.h"

#include <iostream>
#include <vector>
#include <memory>
#include "gmm_gmm.h"

namespace flow
{

/**
 * @brief Picard solver
 * @authors Adrien Crovato
 */
class FLOW_API Picard : public Solver
{
public:
    double relax; ///< relaxation

public:
    Picard(std::shared_ptr<Problem> _pbl);
    ~Picard();

    virtual bool run() override;

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    void build(gmm::csr_matrix<double> &A, std::vector<double> &b);
};

} // namespace flow

#endif //WPICARD_H
