/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WSOLVER_H
#define WSOLVER_H

#include "heat.h"
#include "wObject.h"
#include "wTimer.h"
#include "wTimers.h"
#include "wElement.h"
#include <iostream>
#include <vector>
#include <memory>
#include "gmm_gmm.h"

using namespace tbox;

namespace heat
{

/**
 * @brief heat solver
 */

class HEAT_API Solver : public fwk::wSharedObject
{
public:
    int verbose;
#ifndef SWIG
    DisplayHook *dhook;

#endif
    std::shared_ptr<Problem> pbl;

    bool stopit;            ///< stop the calculation
    int nthreads;           ///< # of threads for the assembly
    fwk::Timers tms;        ///< timers
    std::vector<double> T1; ///< nodal temperatures
    double restol;          ///< tolerance on the residual
    int itmax;              ///< max number of iterations
    bool save;              ///< calculate all results
public:
    Solver(std::shared_ptr<Problem> _pbl);
    virtual ~Solver();

    virtual void start(std::shared_ptr<MshExport> mshWriter);

    void setGUI(DisplayHook &hook) { dhook = &hook; }

    void stop() { stopit = true; }
#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif

private:
    void buildK(gmm::csr_matrix<double> &K2, std::vector<double> &rhs);
    void builds(std::vector<double> &s);
    void buildq(std::vector<double> &s);
    void buildqint(std::vector<double> &qint);
};

} // namespace heat

#endif //WSOLVER_H
