#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from __future__ import division
from past.utils import old_div
import heat as h
import heat.utils as u
import tbox
import tbox.gmsh as gmsh
from fwk.testing import *

def main(**d):
    # default params
    p={}
    p['iR'] = 1.0
    p.update(d) 

    msh = gmsh.MeshLoader("mat2hole.geo",__file__).execute(**p)
    gmshWriter = tbox.GmshExport(msh)

    pbl = h.Problem(msh)

    pbl.add(h.Medium(msh, "Matrix", tbox.Fct2C(1e-5,1e-5,0)))
    pbl.add(h.Medium(msh, "Inclusion", tbox.Fct2C(1e5,1e5,0)))
    
    pbl.add(h.Source(msh, "Inclusion", tbox.Fct0C(1e-4)))

    gleft = h.Dirichlet(msh,"Left", 3.)
    pbl.add(gleft)
    gright = h.Dirichlet(msh,"Right",5.)
    pbl.add(gright)

    # solve problem
    solver = u.solve(pbl, gmshWriter)

    # extract some results
    x1,y1,T1 = u.getTemp(solver,"Top", sort='x')
    
    u.simpleplot(x1, T1, xlabel='x [mm]', title='Temperature along x')
       
    tests = CTests()
    tests.add(CTest('mean(Ttop)', old_div(sum(T1),len(T1)), 6.639428, 1e-3))
    tests.add(CTest('max(Ttop)', max(T1), 8.3445125, 1e-3))
    tests.add(CTest('min(Ttop)', min(T1), 3.0))
    tests.run()

if __name__ == "__main__":
    main()
