#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# test periodic BCs on a honeycomb structure

from __future__ import division
from past.utils import old_div
import heat as h
import heat.utils as u
import tbox
import tbox.gmsh as gmsh
from fwk.testing import *

class MyMaterial(tbox.Fct2U):
    def __init__(self):
        tbox.Fct2U.__init__(self)
    def eval(self, u, k, fake):
        t1 = -5.
        t2 = 45.
        k1 = 1.
        k2 = 100.
        k[0,0] = k[1,1] = k1+(k2-k1)/(t2-t1)*(u-t1)
        #k[0,0] = k[1,1] = k1 # simple case
        #print "returning k=%f" %k
        return k

def main():
    # mesh a unit square
    pars={ 'h' : 10. , 'k' : 1., 'lc' : old_div(1.,3)}
    msh = gmsh.MeshLoader("../../models/honeycomb.geo",__file__).execute(**pars)
    gmshWriter = tbox.GmshExport(msh)
    pbl = h.Problem(msh)

    pbl.add(h.Medium(msh, "Hexagon", tbox.Fct2C(10,10,0), 1.0))
    f = MyMaterial()
    pbl.add(h.Medium(msh, "Borders", f, 1.0))

    TM   = 20.
    dTxM = 1.
    dTyM = 1.
    p = h.Periodic(pbl, TM, dTxM, dTyM)
    pbl.add(p)
    
    # solve problem
    solver = u.solve(pbl, gmshWriter) 

    # extract some results            
    tests = CTests()
    tests.add(CTest('mean flux X', p.qM.x[0], -12.6239, 1e-4))
    tests.add(CTest('mean flux Y', p.qM.x[1], -12.4904, 1e-4))
    tests.run()




if __name__ == "__main__":
    main()
