SetFactory("OpenCASCADE");
If(GMSH_MAJOR_VERSION!=3 || GMSH_MINOR_VERSION!=0 || GMSH_PATCH_VERSION!=6)
  Printf("gmsh %g.%g.%g is not supported by this .geo",GMSH_MAJOR_VERSION,GMSH_MINOR_VERSION,GMSH_PATCH_VERSION);
Else
  LX = 10;
  LY = 10;
  lx2 = 2;
  ly1 = 2;
  ly2 = 7;
  LZ = 1;

  lc = 0.1;

  nX1 = 5;
  nY1 = 3;
  nX2 = 3;
  nX3 = 4;
  nY2 = 6;  
  nZ = 1;

  Point(1)  = { 0.,  LY-ly1, 0.,     lc};
  Point(2)  = { 0.,  LY, 0.,     lc};
  Point(3)  = { LX-lx2,  LY, 0.,     lc};
  Point(4)  = { LX-lx2,  LY-ly1, 0.,     lc};
  Point(5)  = { LX,  LY, 0.,     lc};
  Point(6)  = { LX,  LY-ly1, 0.,     lc};  

  Point(7)  = { LX-lx2,  0., 0.,     lc};
  Point(8)  = { LX-lx2,  ly2, 0.,     lc};
  Point(9)  = { LX,  ly2, 0.,     lc};
  Point(10)  = { LX,  0., 0.,     lc};

  Line(1) = {1, 2};
  Line(2) = {2, 3};
  Line(3) = {3, 4};
  Line(4) = {4, 1};
  Line(5) = {3, 5};
  Line(6) = {5, 6};
  Line(7) = {6, 4};

  Line(8) = {7, 8};
  Line(9) = {8, 9};
  Line(10) = {9, 10};
  Line(11) = {10, 7};  

  Transfinite Line {1, 3, 6} = nY1 Using Progression 1;
  Transfinite Line {2, 4} = nX1 Using Progression 1;
  Transfinite Line {5, 7} = nX3 Using Progression 1;

  Transfinite Line {8, 10} = nY2 Using Progression 1;
  Transfinite Line {9, 11} = nX2 Using Progression 1;

  Line Loop(9) = {1, 2, 3, 4};
  Plane Surface(9) = {9};
  Transfinite Surface {9};
  Recombine Surface {9};

  Line Loop(10) = {-3, 5, 6, 7};
  Plane Surface(10) = {10};
  Transfinite Surface {10};
  Recombine Surface {10};  

  Line Loop(12) = {8, 9, 10, 11};
  Plane Surface(12) = {12};
  Transfinite Surface {12};
  Recombine Surface {12};  

  Extrude {0, 0, LZ}
  {
    Surface{9,10,12}; Layers{nZ}; Recombine;
  }

  Physical Volume("body") = {1,2,3};

  Physical Surface("load 1") = {14};
  Physical Surface("load 2") = {20};
  Physical Surface("clamped 1") = {13};
  Physical Surface("clamped 2") = {25};
  Physical Surface("conctact 1") = {18};
  Physical Surface("conctact 2") = {23};
EndIf
