#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
from katropton.readers import *


def read_iter_all_ensemble_sizes(n_samples, ensemble_sizes, file_MC_base_dir, reduction=False):
    iter = np.zeros((len(ensemble_sizes), n_samples))
    for i in range(0, len(ensemble_sizes)):
        file_MC_base_dir_2 = file_MC_base_dir + \
            '/' + str(ensemble_sizes[i])+'/'
        for j in range(0, int(old_div(n_samples, ensemble_sizes[i]))):
            input_MC_file_dir = file_MC_base_dir_2 + str(j)+'/'
            es = ensemble_sizes[i]
            if reduction:
                es = 1
            iter_indices, residuals = read_Belos(
                input_MC_file_dir + 'belos_out.txt', es)
            for k in range(0, ensemble_sizes[i]):
                iter[i][j*ensemble_sizes[i]+k] = iter_indices[-1]
    return iter


def plot_iter_all_ensemble_sizes(n_samples, iter):
    import matplotlib.pyplot as plt
    plt.figure()
    plt.plot(list(range(0, n_samples)),
             iter[0][:], 'bo', markersize=8, zorder=1)
    plt.plot(list(range(0, n_samples)),
             iter[1][:], 'ro', markersize=6, zorder=2)
    plt.plot(list(range(0, n_samples)),
             iter[2][:], 'go', markersize=4, zorder=2)
    plt.plot(list(range(0, n_samples)),
             iter[3][:], 'ko', markersize=2, zorder=3)
    plt.xlabel('sample')
    plt.ylabel('Number of iterations to converge')


def main():
    import matplotlib.pyplot as plt

    n_samples = 960
    ensemble_sizes = [1, 8, 16, 32]
    for case_id in range(1, 5):
        file_MC_base_dir = '/Volumes/HD_SONY/CMAME/Plate/katropton_CMAME_tests_plate_MC_GS_case_' + \
            str(case_id)+'_ER'
        try:
            iter = read_iter_all_ensemble_sizes(
                n_samples, ensemble_sizes, file_MC_base_dir, True)
            plot_iter_all_ensemble_sizes(n_samples, iter)
            plt.savefig('GS_case_'+str(case_id)+'_ER.png', dpi=400)
            np.savetxt('GS_case_'+str(case_id)+'_ER.txt', iter, delimiter=' ')
        except:
            print(file_MC_base_dir + ' has not all the files')
        file_MC_base_dir = '/Volumes/HD_SONY/CMAME/Plate/katropton_CMAME_tests_plate_MC_direct_case_' + \
            str(case_id)+'_ER'
        try:
            iter = read_iter_all_ensemble_sizes(
                n_samples, ensemble_sizes, file_MC_base_dir, True)
            plot_iter_all_ensemble_sizes(n_samples, iter)
            plt.savefig('direct_case_'+str(case_id)+'_ER.png', dpi=400)
            np.savetxt('direct_case_'+str(case_id) +
                       '_ER.txt', iter, delimiter=' ')
        except:
            print(file_MC_base_dir + ' has not all the files')
        file_MC_base_dir = '/Volumes/HD_SONY/CMAME/Plate/katropton_CMAME_tests_plate_MC_GS_case_' + \
            str(case_id)+'_NER'
        try:
            iter = read_iter_all_ensemble_sizes(
                n_samples, ensemble_sizes, file_MC_base_dir)
            plot_iter_all_ensemble_sizes(n_samples, iter)
            plt.savefig('GS_case_'+str(case_id)+'_NER.png', dpi=400)
            np.savetxt('GS_case_'+str(case_id)+'_NER.txt', iter, delimiter=' ')
        except:
            print(file_MC_base_dir + ' has not all the files')
        file_MC_base_dir = '/Volumes/HD_SONY/CMAME/Plate/katropton_CMAME_tests_plate_MC_direct_case_' + \
            str(case_id)+'_NER'
        try:
            iter = read_iter_all_ensemble_sizes(
                n_samples, ensemble_sizes, file_MC_base_dir)
            plot_iter_all_ensemble_sizes(n_samples, iter)
            plt.savefig('direct_case_'+str(case_id)+'_NER.png', dpi=400)
            np.savetxt('direct_case_'+str(case_id) +
                       '_NER.txt', iter, delimiter=' ')
        except:
            print(file_MC_base_dir + ' has not all the files')


if __name__ == "__main__":
    main()
