#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
import katropton.CMAME.model as model

import katropton as m
import tbox
import tbox.gmsh as gmsh
import shlex
import subprocess
import os
import numpy as np

from fwk.wutils import parseargs

from PyTrilinos import Teuchos

import katropton.utilities as u


def evaluate_one_ensemble(ensemble_size, randomVariable, msh, comm, file_dir):

    work_dir = os.getcwd()
    args = parseargs()

    p = model.defaut_params
    p['Dispersion level'] = 0.1
    #p['gamma'] = '{4663653.515942637,5000307.969231523,5125491.581727843,4650754.5340228975,4802461.555172057,4330669.2615991,4289095.323914186,4521634.142120181}'
    p['MueLu xml template file name'] = '/../preconditioners/SIMPLE_gs_direct.xml.in'
    p['MueLu xml params name'] = ['$BLOCK_00_STRIDING_INFO', '$BLOCK_11_STRIDING_INFO',
                                  '$BLOCK_00_GS_sweeps', '$BLOCK_00_GS_damping',
                                  '$SCHUR_omega', '$SimpleSmoother_damping']
    p['MueLu xml params'] = ['3', '3', '100', '1.75', '0.05', '1.']

    #p['inner clamped'] = True

    #p['Write txt files'] = True
    #p['Write matrix and vectors'] = True

    pm = model.plate_model(ensemble_size, randomVariable,
                           msh, comm, file_dir, work_dir, p)
    pm.run(args.k)


def evaluate_all_ensembles(ensemble_size, randomVariable, msh, comm, file_dir):

    directory = str(ensemble_size)
    if not os.path.exists(directory):
        os.makedirs(directory)
    os.chdir(directory)

    N = randomVariable.shape[0]
    N_ensemble = old_div(N, ensemble_size)
    for ensemble_index in range(0, N_ensemble):
        directory = str(ensemble_index)
        if not os.path.exists(directory):
            os.makedirs(directory)
        os.chdir(directory)

        evaluate_one_ensemble(ensemble_size, randomVariable[ensemble_index:(
            ensemble_index+ensemble_size), :], msh, comm, file_dir)

        os.chdir('..')

    os.chdir('..')


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
        print("info: MPI found")
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"
        print("info: MPI not found => MPI disabled")
    args = parseargs()

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    # Units in the meshes are in mm
    msh = u.fuse_meshes('plate_1.geo', 'plate_1_tmp', 'plate_2.geo',
                        'plate_2_tmp', 'plate_fused', file_dir, work_dir)

    np.random.seed(42)
    N = 8
    num_random_variables = 3
    randomVariable = np.random.normal(0, 1, num_random_variables*N)
    randomVariable = np.resize(randomVariable, (N, num_random_variables))

    np.savetxt('randomVariable.txt', randomVariable)

    evaluate_all_ensembles(1, randomVariable, msh, comm, file_dir)
    evaluate_all_ensembles(8, randomVariable, msh, comm, file_dir)


if __name__ == "__main__":
    main()
