#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from builtins import str
from builtins import range
import katropton as m
import tbox
import tbox.gmsh as gmsh


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
        print("info: MPI found")
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"
        print("info: MPI not found => MPI disabled")

    from fwk.wutils import parseargs
    args = parseargs()

    from PyTrilinos import Teuchos
    import numpy as np

    precondList = Teuchos.ParameterList()
    precondList = Teuchos.ParameterList()
    precondSubList = Teuchos.ParameterList()
    precondSubSubList = Teuchos.ParameterList()
    precondSubSubList['fact: iluk level-of-fill'] = 2
    precondSubList['Prec Type'] = "RILUK"
    precondSubList['Overlap'] = 1
    precondSubList['Ifpack2 Settings'] = precondSubSubList
    precondList['Ifpack2'] = precondSubList

    mueluParams = Teuchos.ParameterList()
    #mueluParamsSub1 = Teuchos.ParameterList()
    #mueluParamsSub1['PDE equations'] = 4
    #mueluParams['Matrix'] = mueluParamsSub1
    mueluParams['number of equations'] = 4
    mueluParams['max levels'] = 2
    mueluParams['multigrid algorithm'] = "unsmoothed"
    mueluParams['smoother: type'] = "RELAXATION"

    mueluParamsSub2 = Teuchos.ParameterList()

    mueluParamsSub2['relaxation: type'] = "Symmetric Gauss-Seidel"
    mueluParamsSub2['relaxation: sweeps'] = 4
    mueluParamsSub2['relaxation: damping factor'] = 1.25

    mueluParams['coarse: type'] = "RELAXATION"

    mueluParamsSub3 = Teuchos.ParameterList()

    mueluParamsSub3['relaxation: type'] = "Symmetric Gauss-Seidel"
    mueluParamsSub3['relaxation: sweeps'] = 4
    mueluParamsSub3['relaxation: damping factor'] = 1.25

    mueluParams['smoother: params'] = mueluParamsSub2
    mueluParams['coarse: params'] = mueluParamsSub3

    solverList = Teuchos.ParameterList()
    solverList['type'] = "BlockGmres"
    solverList['Convergence Tolerance'] = 10**(-8)
    solverList['Maximum Iterations'] = 1000
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 100
    solverList['ifpackParams'] = precondList
    solverList['mueluParams'] = mueluParams

    solverList['Use preconditioner'] = True

    solverList['Maximum active set iteration'] = 10

    solverList['Print Teuchos timers'] = False
    #solverList['Write matrix and vectors'] = True
    #solverList['Write txt files'] = True

    geo_name = 'M1_FEM_7.msh'
    mesh_name = 'M1_FEM_7.msh'

    import shlex
    import subprocess
    import os

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    #solverList["MueLu xml file name"] = file_dir + '/finest_level_prec_2.xml'
    solverList["MueLu xml file name"] = file_dir + \
        '/finest_level_prec_simple_heat.xml'
    #solverList["MueLu xml file name"] = file_dir + '/nested_multigrid_2_lvls_chebyshev_heat.xml'
    #solverList["MueLu xml file name"] = file_dir + '/nested_multigrid_2_lvls_chebyshev.xml'

    if rank == 0:
        command_line = 'gmsh -3 ' + file_dir + '/mesh/' + geo_name + \
            ' -o ' + work_dir + '/' + mesh_name + ' -part ' + str(siz)
        print(command_line)
        tmp = shlex.split(command_line)
        fileout = open('gmsh.log', 'w')
        p = subprocess.Popen(tmp, stdin=subprocess.PIPE, stdout=fileout,
                             stderr=fileout, env=os.environ, shell=False, close_fds=True)
        retcode = p.wait()
        for i in range(1, siz):
            comm.send(1, dest=i, tag=11)
    else:
        data = comm.recv(source=0, tag=11)

    if siz == 1:
        msh = gmsh.MeshLoader(mesh_name, work_dir).execute()
    else:
        msh = gmsh.MeshLoader(mesh_name, work_dir).myrank_execute(rank)

    pbl = m.Problem(msh, comm)

    ensemble_size = 1

    E = 330000 * np.ones(ensemble_size)
    nu = 0.36 * np.ones(ensemble_size)
    k = 0.114
    d = 0.5E-05

    E_rhodium = 379000 * np.ones(ensemble_size)
    nu_rhodium = 0.26 * np.ones(ensemble_size)
    k_rhodium = 0.150
    d_rhodium = 0.781E-05

    E_WCu = 280000 * np.ones(ensemble_size)
    nu_WCu = 0.26 * np.ones(ensemble_size)
    k_WCu = 0.180
    d_WCu = 0.88E-05

    E_SS = 180000 * np.ones(ensemble_size)
    nu_SS = 0.26 * np.ones(ensemble_size)
    k_SS = 0.020
    d_SS = 1.6E-05

    E_CuCrZr = 130000 * np.ones(ensemble_size)
    nu_CuCrZr = 0.26 * np.ones(ensemble_size)
    k_CuCrZr = 0.280
    d_CuCrZr = 1.6E-05

    E_I718 = 180000 * np.ones(ensemble_size)
    nu_I718 = 0.26 * np.ones(ensemble_size)
    k_I718 = 0.020
    d_I718 = 1.4E-05

    m.Medium(pbl, "Mirror", "Rhodium", E_rhodium,
             nu_rhodium, k_rhodium, d_rhodium)
    m.Medium(pbl, "Substrate", "WCu", E_WCu, nu_WCu, k_WCu, d_WCu)
    m.Medium(pbl, "Shaft 1", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Shaft 2", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Shaft 3", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Holder", "SS", E_SS, nu_SS, k_SS, d_SS)

    m.Medium(pbl, "Spacer 1-2", "CuCrZr", E_CuCrZr,
             nu_CuCrZr, k_CuCrZr, d_CuCrZr)
    m.Medium(pbl, "Spacer 3", "CuCrZr", E_CuCrZr,
             nu_CuCrZr, k_CuCrZr, d_CuCrZr)

    m.Medium(pbl, "Spacer 1-2 part 2", "CuCrZr",
             E_CuCrZr, nu_CuCrZr, k_CuCrZr, d_CuCrZr)
    m.Medium(pbl, "Spacer 3 part 2", "CuCrZr",
             E_CuCrZr, nu_CuCrZr, k_CuCrZr, d_CuCrZr)

    m.Medium(pbl, "Washer 1", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Washer 2", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Washer 3", "I718", E_I718, nu_I718, k_I718, d_I718)

    m.Medium(pbl, "Nut 1", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Nut 2", "I718", E_I718, nu_I718, k_I718, d_I718)
    m.Medium(pbl, "Nut 3", "I718", E_I718, nu_I718, k_I718, d_I718)

    m.Dirichlet(pbl, "Holder Clamping", "Clamped", 1,
                 0., 1, 0., 1, 0., 0, 0., ensemble_size)

    m.Dirichlet(pbl, "Cooling top", "Clamped", 0, 0.,
                 0, 0., 0, 0., 1, 70., ensemble_size)
    m.Dirichlet(pbl, "Cooling bot", "Clamped", 0, 0.,
                 0, 0., 0, 0., 1, 70., ensemble_size)

    norm1 = tbox.Pt(0, 0, -1)
    norm2 = tbox.Pt(0, 0, 1)

    cont1 = m.Contact(pbl, "Nut 1 to Holder", "contact", norm1, -0.3)
    cont1.setMaster(pbl, "Holder to Nut 1", norm1)

    cont2 = m.Contact(pbl, "Nut 2 to Holder", "contact", norm1, -0.3)
    cont2.setMaster(pbl, "Holder to Nut 2", norm1)

    cont3 = m.Contact(pbl, "Nut 3 to Holder", "contact", norm1, -0.3)
    cont3.setMaster(pbl, "Holder to Nut 3", norm1)

    cont4 = m.Contact(pbl, "Spacer 1-2 to Substrate", "contact", norm1, 0.)
    cont4.setMaster(pbl, "Substrate to Spacer 1-2", norm1)

    cont5 = m.Contact(pbl, "Spacer 3 to Substrate", "contact", norm1, 0.)
    cont5.setMaster(pbl, "Substrate to Spacer 3", norm1)

    cont6 = m.Contact(pbl, "Spacer 1-2 part 2 to Holder", "contact", norm1, 0.)
    cont6.setMaster(pbl, "Holder to Spacer 1-2 part 2", norm1)

    cont7 = m.Contact(pbl, "Spacer 3 part 2 to Holder", "contact", norm1, 0.)
    cont7.setMaster(pbl, "Holder to Spacer 3 part 2", norm1)

    cont8 = m.Contact(pbl, "Spacer 1-2 to Spacer 1-2 part 2",
                      "contact", norm1, 0.)
    cont8.setMaster(pbl, "Spacer 1-2 part 2 to Spacer 1-2", norm1)

    cont9 = m.Contact(pbl, "Spacer 3 to Spacer 3 part 2", "contact", norm1, 0.)
    cont9.setMaster(pbl, "Spacer 3 part 2 to Spacer 3", norm1)

    cont10 = m.Contact(pbl, "Spacer 1-2 to Washer 1", "contact", norm1, 0.)
    cont10.setMaster(pbl, "Washer 1 to Spacer 1-2", norm1)
    # cont10.setInitialyOpen()

    cont11 = m.Contact(pbl, "Washer 1 to Spacer 1-2 part 2",
                       "contact", norm1, 0.)
    cont11.setMaster(pbl, "Spacer 1-2 part 2 to Washer 1", norm1)
    # cont11.setInitialyOpen()

    cont12 = m.Contact(pbl, "Washer 1 to Shaft 1", "contact", norm1, 0.)
    cont12.setMaster(pbl, "Shaft 1 to Washer 1", norm1)
    # cont12.setInitialyOpen()

    cont13 = m.Contact(pbl, "Spacer 1-2 to Washer 2", "contact", norm1, 0.)
    cont13.setMaster(pbl, "Washer 2 to Spacer 1-2", norm1)
    # cont13.setInitialyOpen()

    cont14 = m.Contact(pbl, "Washer 2 to Spacer 1-2 part 2",
                       "contact", norm1, 0.)
    cont14.setMaster(pbl, "Spacer 1-2 part 2 to Washer 2", norm1)
    # cont14.setInitialyOpen()

    cont15 = m.Contact(pbl, "Shaft 2 to Washer 2", "contact", norm1, 0.)
    cont15.setMaster(pbl, "Washer 2 to Shaft 2", norm1)
    # cont15.setInitialyOpen()

    cont16 = m.Contact(pbl, "Spacer 3 to Washer 3", "contact", norm1, 0.)
    cont16.setMaster(pbl, "Washer 3 to Spacer 3", norm1)
    # cont16.setInitialyOpen()

    cont17 = m.Contact(pbl, "Washer 3 to Spacer 3 part 2",
                       "contact", norm1, 0.)
    cont17.setMaster(pbl, "Spacer 3 part 2 to Washer 3", norm1)
    # cont17.setInitialyOpen()

    cont18 = m.Contact(pbl, "Washer 3 to Shaft 3", "contact", norm1, 0.)
    cont18.setMaster(pbl, "Shaft 3 to Washer 3", norm1)
    # cont18.setInitialyOpen()

    #m.Dirichlet(pbl, "Nut 1 to Holder", "Clamped",1,0.,1,0.,1,0.,1,100.,ensemble_size)
    #m.Dirichlet(pbl, "Nut 2 to Holder", "Clamped",1,0.,1,0.,1,0.,1,100.,ensemble_size)
    #m.Dirichlet(pbl, "Nut 3 to Holder", "Clamped",1,0.,1,0.,1,0.,1,100.,ensemble_size)

    #m.Dirichlet(pbl, "Holder to Nut 1", "Clamped",1,0.,1,0.,1,0.,1,100.,ensemble_size)
    #m.Dirichlet(pbl, "Holder to Nut 2", "Clamped",1,0.,1,0.,1,0.,1,100.,ensemble_size)
    #m.Dirichlet(pbl, "Holder to Nut 3", "Clamped",1,0.,1,0.,1,0.,1,100.,ensemble_size)

    #m.Dirichlet(pbl, "Washer 1 to Spacer 1-2", "Clamped",1,0.,1,0.,1,0.,1,70.,ensemble_size)
    #m.Dirichlet(pbl, "Washer 2 to Spacer 1-2", "Clamped",1,0.,1,0.,1,0.,1,70.,ensemble_size)
    #m.Dirichlet(pbl, "Washer 3 to Spacer 3", "Clamped",1,0.,1,0.,1,0.,1,70.,ensemble_size)

    #m.Dirichlet(pbl, "Spacer 1-2 to Substrate", "Clamped",1,0.,1,0.,1,0.,1,70.,ensemble_size)
    #m.Dirichlet(pbl, "Spacer 3 to Substrate", "Clamped",1,0.,1,0.,1,0.,1,70.,ensemble_size)

    #m.Dirichlet(pbl, "Spacer 1-2 part 2 to Holder", "Clamped",1,0.,1,0.,1,0.,1,70.,ensemble_size)
    #m.Dirichlet(pbl, "Spacer 3 part 2 to Holder", "Clamped",1,0.,1,0.,1,0.,1,70.,ensemble_size)

    zero = np.zeros(ensemble_size)
    hf = (70/0.0121)*0.001 * np.ones(ensemble_size)

    m.Neumann(pbl, "Mirror surface", "moved", 0, zero,
              0, zero, 0, zero, 1, hf, ensemble_size)
    m.Source(pbl, "Mirror", "ihg", 2.45*1000)
    m.Source(pbl, "Substrate", "ihg", 2.45*1000)
    m.Source(pbl, "Holder", "ihg", 0.7*1000)

    m.Dirichlet(pbl, "XY Clamping", "Clamped", 1, 0.,
                 1, 0., 0, 0., 0, 0., ensemble_size)
    #m.Dirichlet(pbl, "Mirror surface", "Clamped",0,0.,0,0.,1,-0.1,1,300.,ensemble_size)

    slv = m.IterativeSolver(pbl, args.k, solverList, 4, ensemble_size)

    if not args.nogui:
        if rank == 0:
            all_msh = gmsh.MeshLoader(mesh_name, work_dir).execute()
            import katropton.viewer as v
            gui = v.MeshViewer(pbl, slv, all_msh)
            gui.vmin = 0
            gui.vmax = 1
            gui.start()
        else:
            slv.start()
    else:
        slv.start()

    from fwk.testing import *
    tests = CTests()
    tests.add(CTest('mean(Tbottom)', 0, 0))
    tests.run()


if __name__ == "__main__":
    main()
