#include "ElementsList.h"

#include "wProblem.h"
#include "wTag.h"
#include "wMshData.h"
#include "wElement.h"
#include "wNode.h"
#include "wMedium.h"

using namespace katropton;

/**
 * @brief ElementsList constructor
 * 
 * This constructor allocates 1 Kokkos::View which stores all the required information related to the elements, including:
 * <ul>
 *  <li> all their local node IDs,
 *  <li> their size, type, and material (at most one material per element).
 * </ul>
 *
 * Arguments:
 * <ul>
 *  <li> pbl: a Problem object,
 *  <li> map: an RCP to a Map object.
 * </ul>
 */
ElementsList::ElementsList(Problem &pbl, Teuchos::RCP<Map> map)
{
    typedef Map::local_ordinal_type local_ordinal_type;

    elements_number = map->mapElems->getNodeNumElements();

    local_ordinal_type materials_number = pbl.media.size();
    elements_type = 0;
    elements_size = 1;
    elements_material = 2;
    elements_nodes = 3;

    elements = Kokkos::View<int **, Kokkos::LayoutRight>("R", elements_number, elements_nodes + 8);

    int i1 = 0;
    int i2 = 0;
    while (i1 < elements_number)
    {
        tbox::Element *e = pbl.msh->elems[i2++];
        if (map->mapElems->isNodeGlobalElement(e->no - 1))
        {
            elements(i1, elements_type) = static_cast<int>(e->type());
            elements(i1, elements_size) = e->nodes.size();
            for (auto j = 0; j < materials_number; ++j)
                if (e->ptag == (pbl.media[j]->tag))
                {
                    elements(i1, elements_material) = j;
                    break;
                }
            for (auto j = 0; j < e->nodes.size(); ++j)
                elements(i1, elements_nodes + j) = map->mapNodesWO->getLocalElement(e->nodes[j]->row);
            ++i1;
        }
    }
}
