#include "wProblem.h"
#include "wMshData.h"

#include "wDirichlet.h"
#include "wNeumann.h"
#include "wSource.h"
using namespace katropton;
using namespace tbox;

/**
 * @brief Problem constructor
 * 
 * Arguments:
 * <ul>
 *  <li> _msh: a shared pointer to the mesh,
 *  <li> _comm: the used MPI communicator.
 * </ul>
 */
Problem::Problem(std::shared_ptr<MshData> _msh, MPI_Comm _comm) : msh(_msh), comm(_comm)
{
}

/**
 * @brief Add a random variable.
 */
void Problem::add(std::shared_ptr<RandomVariable> r)
{
    RandomVariables.push_back(r);
}

/**
 * @brief Display information.
 */
void Problem::write(std::ostream &out) const
{
    out << "katropton::Problem:\n";
    out << " + mesh:\n";
    out << *msh; //wMshData.h

    out << " + dBC:\n";
    for (auto b : duBCs)
        out << *b << '\n';
    out << " + nBC:\n";
    for (auto b : nBCs)
        out << *b << '\n';
    out << " + Source:\n";
    for (auto b : Sources)
        out << *b << '\n';
}