#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from builtins import str
from builtins import range
import katropton as m
import tbox
import tbox.gmsh as gmsh
import katropton.utilities as u
#import matplotlib.pyplot as plt


def main():
    comm, rank, size = u.import_MPI()

    from PyTrilinos import Teuchos

    import shlex
    import subprocess
    import os
    import numpy as np

    geo_name = '2_cubes_patch.geo'
    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    msh, mesh_name, partitioned_mesh_name = u.read_mesh_MPI(
        geo_name, file_dir, work_dir, comm, rank, size)

    ensemble_size = 1

    pbl = m.Problem(msh, comm)

    #E = np.linspace(0.5,1.5,ensemble_size)
    E = 10. * np.ones(ensemble_size)
    nu = 0.4 * np.ones(ensemble_size)
    k = 0.114
    d = 0.
    m.Medium(pbl, "Body 1", "test", E, nu, k, d)
    m.Medium(pbl, "Body 2", "test", E, nu, k, d)

    m.Dirichlet(pbl, "Clamped", "Clamped", 1, 0.,
                1, 0., 1, 0., 0, 0., ensemble_size)
    #m.Dirichlet(pbl, "Clamped xy", "Clamped",1,0.,1,0.,1,0.,0,0.,ensemble_size)

    norm = tbox.Pt(0, 0, -1)
    cont = m.Contact(pbl, "Contact 2", "contact", norm)
    cont.setMeshTying()
    cont.setMaster(pbl, "Contact 1", norm)

    zero = np.zeros(ensemble_size)
    p = -0.2 * np.ones(ensemble_size)

    m.Neumann(pbl, "Load", "Load 1", 0, zero, 0,
              zero, 1, p, 0, zero, ensemble_size)

    solverList = Teuchos.ParameterList()
    solverList['Convergence Tolerance'] = 10**(-8)
    solverList['Maximum Iterations'] = 5000
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 1

    solverList["MueLu xml file name"] = file_dir + \
        '/../preconditioners/SIMPLEC_mtgs.xml'
    solverList["convert MueLu xml file"] = True

    solverList['Use preconditioner'] = True

    solverList['type'] = "BlockGmres"
    solverList['Maximum active set iteration'] = 1

    solverList['Print Teuchos timers'] = True
    #solverList['Write matrix and vectors'] = True
    solverList['Write txt files'] = True

    if ensemble_size > 1:
        slv = m.IterativeSolver(pbl, solverList,
                                3, True, ensemble_size)
    else:
        slv = m.IterativeSolver(pbl, solverList,
                                3, ensemble_size)

    nThreads = u.Initialize_Kokkos()

    slv.start()

    u.Finalize_Kokkos()


if __name__ == "__main__":
    main()
