#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import print_function
from builtins import str
from builtins import range
import time
import mirrors as m
import tbox
import tbox.gmsh as gmsh
import fwk.wutils as wu
from fwk.coloring import ccolors

def main():

    print(ccolors.ANSI_RED + "One of the meshes contains inverted Tri3 elements. Test will fail!" + ccolors.ANSI_RESET)

    pbl_vector = []

    #tests descriptions/parameters
    
    from fwk.wutils import parseargs
    args=parseargs()
    
    nb_of_process = args.k

    print_mesh_info = 0
    print_problem_info = 0
    print_solver_info = 0
    print_tms = 1
    print_diff = 1

    clamped = tbox.Pt(1,1,1)
    xclamped = tbox.Pt(1,0,0)
    yclamped = tbox.Pt(0,1,0)
    zclamped = tbox.Pt(0,0,1)

    values_clamped = tbox.Pt(0,0,0)

    forces = tbox.Pt(-0.2,0,0)

    #----------------test 0----------------

    loader = gmsh.MeshLoader("small_mesh.msh",__file__)
    msh0 = loader.execute()
        
    pbl = m.Problem(msh0,'thermomechanical',22.)
    #m.Medium(pbl, "Volume", "CuCrZr",2000.,0.,0.114,0.5E-05)
    m.Medium(pbl, "Volume", "TZM",330000,0.36,0.114,0.5E-05)

    m.ANSYSSolution(pbl,'T','T_ansys',wu.pth('small_T.ansys',__file__))
    m.ANSYSSolution(pbl,'u','u_ansys_thermomech',wu.pth('small_u.ansys',__file__))
    m.uDirichlet(pbl, "X0", "Clamped",clamped,values_clamped)
    m.TDirichlet(pbl, "X0", "Clamped",22.)
    m.TDirichlet(pbl, "X1", "Clamped",100.)

    pbl_vector.append(pbl)

    #----------------test 1----------------

    pbl = m.Problem(msh0,'thermomechanical',22.)
    #m.Medium(pbl, "Volume", "CuCrZr",2000.,0.,0.114,0.5E-05)
    m.Medium(pbl, "Volume", "TZM",330000,0.36,0.114,0.5E-05)

    m.ANSYSSolution(pbl,'T','T_ansys',wu.pth('small2_T.ansys',__file__))
    m.ANSYSSolution(pbl,'u','u_ansys_thermomech',wu.pth('small2_u.ansys',__file__))
    m.ANSYSSolution(pbl,'x','x_ansys_thermomech',wu.pth('small2_x.ansys',__file__))
    m.ANSYSSolution(pbl,'y','y_ansys_thermomech',wu.pth('small2_y.ansys',__file__))
    m.ANSYSSolution(pbl,'z','z_ansys_thermomech',wu.pth('small2_z.ansys',__file__))
    m.uDirichlet(pbl, "X0", "xClamped",xclamped,values_clamped)
    m.uDirichlet(pbl, "Y0", "yClamped",yclamped,values_clamped)
    m.uDirichlet(pbl, "Z0", "zClamped",zclamped,values_clamped)
    m.TDirichlet(pbl, "X0", "Clamped",100.)
    m.TDirichlet(pbl, "X1", "Clamped",100.)

    pbl_vector.append(pbl)

    #----------------test 2----------------

    loader = gmsh.MeshLoader("mirrors01_ansys.msh",__file__)
    msh2 = loader.execute()    
    
    pbl = m.Problem(msh2,'thermal',22.)
    #m.Medium(pbl, "Volume", "CuCrZr",2000.,0.,0.114,0.5E-05)
    m.Medium(pbl, "Volume", "TZM",330000,0.36,0.114,0.5E-05)

    m.ANSYSSolution(pbl,'T','T_ansys',wu.pth('mirrors01_T.ansys',__file__))
    m.TDirichlet(pbl, "M1_S1", "Clamped",70.)
    m.TDirichlet(pbl, "M1_S2", "Clamped",70.)
    m.TDirichlet(pbl, "M1_S3", "Clamped",70.)
    m.TDirichlet(pbl, "M1_B1", "Clamped",70.)
    m.TDirichlet(pbl, "M1_B2", "Clamped",70.)
    m.TDirichlet(pbl, "M1_B3", "Clamped",70.)
    m.TNeumann(pbl, "HF", "HF",1.E-002)
    m.TSource(pbl, "Volume", "ihg",1.7E-004)

    pbl_vector.append(pbl)

    #----------------test 3----------------

    pbl = m.Problem(msh2,'mechanical',22.)
    #m.Medium(pbl, "Volume", "CuCrZr",2000.,0.,0.114,0.5E-05)
    m.Medium(pbl, "Volume", "TZM",330000,0.36,0.114,0.5E-05)

    m.ANSYSSolution(pbl,'u','u_ansys_mech',wu.pth('mirrors01_u.ansys',__file__))
    m.ANSYSSolution(pbl,'x','x_ansys_mech',wu.pth('mirrors01_x.ansys',__file__))
    m.ANSYSSolution(pbl,'y','y_ansys_mech',wu.pth('mirrors01_y.ansys',__file__))
    m.ANSYSSolution(pbl,'z','z_ansys_mech',wu.pth('mirrors01_z.ansys',__file__))
    m.uDirichlet(pbl, "M1_S1", "Clamped",clamped,values_clamped)
    m.uDirichlet(pbl, "M1_S2", "Clamped",clamped,values_clamped)
    m.uDirichlet(pbl, "M1_S3", "Clamped",clamped,values_clamped)
    m.uDirichlet(pbl, "M1_B1", "Clamped",clamped,values_clamped)
    m.uDirichlet(pbl, "M1_B2", "Clamped",clamped,values_clamped)
    m.uDirichlet(pbl, "M1_B3", "Clamped",clamped,values_clamped)
    m.uNeumann(pbl, "HF", "HF",clamped,forces)

    pbl_vector.append(pbl)

    #----------------test 4----------------

    pbl = m.Problem(msh2,'thermomechanical',22.)
    #m.Medium(pbl, "Volume", "CuCrZr",2000.,0.,0.114,0.5E-05)
    m.Medium(pbl, "Volume", "TZM",330000,0.36,0.114,0.5E-05)

    m.ANSYSSolution(pbl,'T','T_ansys',wu.pth('mirrors01_T.ansys',__file__))
    m.ANSYSSolution(pbl,'u','u_ansys_thermomech',wu.pth('mirrors01_u_thermo.ansys',__file__))
    m.ANSYSSolution(pbl,'x','x_ansys_thermomech',wu.pth('mirrors01_x_thermo.ansys',__file__))
    m.ANSYSSolution(pbl,'y','y_ansys_thermomech',wu.pth('mirrors01_y_thermo.ansys',__file__))
    m.ANSYSSolution(pbl,'z','z_ansys_thermomech',wu.pth('mirrors01_z_thermo.ansys',__file__))
    m.uDirichlet(pbl, "M1_S1", "Clamped",clamped,values_clamped)
    m.uDirichlet(pbl, "M1_S2", "Clamped",clamped,values_clamped)
    m.uDirichlet(pbl, "M1_S3", "Clamped",clamped,values_clamped)
    m.uDirichlet(pbl, "M1_B1", "Clamped",clamped,values_clamped)
    m.uDirichlet(pbl, "M1_B2", "Clamped",clamped,values_clamped)
    m.uDirichlet(pbl, "M1_B3", "Clamped",clamped,values_clamped)
    m.TDirichlet(pbl, "M1_S1", "Clamped",70.)
    m.TDirichlet(pbl, "M1_S2", "Clamped",70.)
    m.TDirichlet(pbl, "M1_S3", "Clamped",70.)
    m.TDirichlet(pbl, "M1_B1", "Clamped",70.)
    m.TDirichlet(pbl, "M1_B2", "Clamped",70.)
    m.TDirichlet(pbl, "M1_B3", "Clamped",70.)
    m.TNeumann(pbl, "HF", "HF",1.E-002)
    m.TSource(pbl, "Volume", "ihg",1.7E-004)

    pbl_vector.append(pbl)

    #tests start
    date = time.strftime("%d/%m/%Y")
    hour = time.strftime("%H:%M:%S")

    f = open('mirrors_test_battery.log','w')
    f.write('Mirrors test battery\n')
    f.write('Tests started at: ' + date + ' ' + hour + '.\n')
    f.write('====================\n')

    for i in range(0,len(pbl_vector)):
        f.write('Test ' + str(i) + ':\n')
        f.write('~~~~~~~\n')

        if print_mesh_info == 1:
            f.write('Mesh informations:\n')
            print(pbl_vector[i].msh, file=f)
            f.write('~ ~ ~ ~ ~ ~ ~\n')

        if print_problem_info == 1:
            f.write('Problem informations:\n')
            print(pbl_vector[i], file=f)
            f.write('~ ~ ~ ~ ~ ~ ~\n')

        solver = m.Solver(pbl_vector[i],nb_of_process)
        if print_solver_info == 1:
            f.write('Solver informations:\n')
            print(solver, file=f)
            f.write('~ ~ ~ ~ ~ ~ ~\n')

        solver.start()

        if print_tms == 1:
            print(solver.tms, file=f)
            f.write('~ ~ ~ ~ ~ ~ ~\n')

        if print_diff ==1:
            for j in range(0,len(pbl_vector[i].ANSYSSol)):
                f.write('ANSYS Solution name: ' + pbl_vector[i].ANSYSSol[j].solution_name + '\n')
                f.write('Absolute error: ' + str(pbl_vector[i].ANSYSSol[j].diff_abs) + '\n')
                f.write('Relative error: ' + str(pbl_vector[i].ANSYSSol[j].diff_rel) + '\n')
                if j < len(pbl_vector[i].ANSYSSol)-1:
                    f.write('~   ~   ~   ~\n')
        if i < len(pbl_vector)-1:
            f.write('--------------------\n')

    #tests are finished
    date2 = time.strftime("%d/%m/%Y")
    hour2 = time.strftime("%H:%M:%S")
    f.write('====================\n')
    f.write('Tests finished at: ' + date2 + ' ' + hour2 + '.\n')

    f.close()
	
if __name__ == "__main__":
    main()
