#!/usr/bin/env python
# -*- coding: utf-8 -*-
# simplified model of a mirror and a holder 
# (two rectangular blocks on top of each other)

import mirrors as m
import tbox
import tbox.gmsh as gmsh

def model(p={}):
    # configure/build mesh
    msh = gmsh.MeshLoader("cube_tri.geo", __file__).execute(**p)
    pbl = m.Problem(msh, "thermomechanical", 0.0)

    # configure pbl
    m.Medium(pbl, "Volume", "Volume", 330000, 0.36, 0.114, 0.5E-05)

    clamped = tbox.Pt(1, 1, 1)
    values_clamped = tbox.Pt(0, 0, 0)

    XYZ = tbox.Pt(1, 1, 1)
    X0Y0Z0 = tbox.Pt(0., 0., 0.)

    X = tbox.Pt(1, 0, 0)
    X1Y0Z0 = tbox.Pt(1.0, 0.0, 0.0)

    #T = p['T']
    m.uDirichlet(pbl, "FaceX0", 
                      "FaceX0", 
                      XYZ, X0Y0Z0)
    m.uDirichlet(pbl, "FaceX1", 
                      "FaceX1", 
                      X, X1Y0Z0)
    m.TDirichlet(pbl, "FaceX1", "FaceX1", 0.)
    m.TDirichlet(pbl, "FaceX0", "FaceX0", 10.)

    return (pbl, msh) # retourner pbl sinon il semble detruit...


def runmodel(**d):
    p = {}
    p['Lx'] = 10.0
    p['Ly'] = 10.0
    p['Lz'] = 2.0
    p['Nx'] = 10
    p['Ny'] = 10
    p['Nz'] = 3
    p.update(d)
    
    # builds an instance of the model with given parameters
    (pbl, msh) = model(p)
    gmshWriter = tbox.GmshExport(msh)

    # retrieves command-line arguments
    from fwk.wutils import parseargs
    args = parseargs()

    # creates a solver and runs it
    solver = m.Solver(pbl, args.k)
    solver.start(gmshWriter)

    if not args.nogui:
        from mirrors.viewer import GUI
        GUI().open('cube_tri')

    return pbl.Sol

if __name__ == "__main__":
    runmodel()
