/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wuNeumann.h"
#include "wProblem.h"
#include "wTag.h"
#include "wElement.h"
#include "wNode.h"
#include <algorithm>
using namespace mirrors;

uNeumann::uNeumann(mirrors::Problem &pbl, int no, std::string _BC_name, Pt _which_dof, Pt _values) : Group(pbl.msh, no), BC_name(_BC_name), which_dof(_which_dof), values(_values)
{
    for (auto e : tag->elems)
        for (auto n : e->nodes)
            nodes.push_back(n);

    pbl.nuBCs.push_back(this);
}

uNeumann::uNeumann(mirrors::Problem &pbl, std::string const &name, std::string _BC_name, Pt _which_dof, Pt _values) : Group(pbl.msh, name), BC_name(_BC_name), which_dof(_which_dof), values(_values)
{
    for (auto e : tag->elems)
        for (auto n : e->nodes)
            nodes.push_back(n);

    pbl.nuBCs.push_back(this);
}

void uNeumann::write(std::ostream &out) const
{
    out << "Mechanical von Neumann BC named ''" << BC_name << "'' on " << *tag << "\n";
    out << "With fixed applied surfacic force (";
    if (which_dof.x[0] == 1)
        out << values.x[0] << ", ";
    else
        out << "-, ";
    if (which_dof.x[1] == 1)
        out << values.x[1] << ", ";
    else
        out << "-, ";
    if (which_dof.x[2] == 1)
        out << values.x[2] << ")\n";
    else
        out << "-)\n";
    out << "a '-' represents the fact that the associated DOF is not under a given force.\n";
    for (auto n : nodes)
    {
        out << '\t' << *n << '\n';
    }
}
