#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from builtins import range
import numpy as np
from numpy.polynomial import Polynomial, Legendre

def next_indices(p,i,M,d):

    i_zero = np.zeros((d,), dtype =np.int)
    i_one = np.ones((d,), dtype =np.int)

    i = i + i_one

    while np.all(i != i_zero):
        i[p] = i[p] + 1;
        if i[p] > M[p]:
            if p==(d-1):
                i= - i_one
                return p, i
            i[p] = 1
            p = p+1
        else:
            p = 0
            return p, i-i_one

def phi(xi,N,n_max,d):

    i =      np.zeros((d,), dtype =np.int)
    i_zero = np.zeros((d,), dtype =np.int)
    i_one =  np.ones((d,), dtype =np.int)
    p = 0

    phi = np.ones((n_max,))
    n_i = 0
    while np.all(i != -i_one):
        for j in range(0,d):
            phi[n_i] = phi[n_i]*xi[j]**i[j]
        n_i = n_i + 1
        p,i = next_indices(p,i,N,d)
    return phi


def phi_2(xi,N,n_max,d):

    i =      np.zeros((d,), dtype =np.int)
    i_zero = np.zeros((d,), dtype =np.int)
    i_one =  np.ones((d,), dtype =np.int)
    p = 0

    phi = np.ones((n_max,))
    n_i = 0
    while np.all(i != -i_one):
        tmp = 0
        for j in range(0,d):
            tmp = tmp + i[j]
        if tmp <= N[0]:
            for j in range(0,d):
                phi[n_i] = phi[n_i]*xi[j]**i[j]
            n_i = n_i + 1
        p,i = next_indices(p,i,N,d)
    return phi


def phi_3(xi,N,n_max,d,Dom_min,Dom_max):

    i =      np.zeros((d,), dtype =np.int)
    i_zero = np.zeros((d,), dtype =np.int)
    i_one =  np.ones((d,), dtype =np.int)
    p = 0

    phi = np.ones((n_max,))
    n_i = 0
    while np.all(i != -i_one):
        for j in range(0,d):
            phi[n_i] = phi[n_i]*Legendre.basis(i[j], [Dom_min[j],Dom_max[j]])(xi[j])
        n_i = n_i + 1
        p,i = next_indices(p,i,N,d)
    return phi

def grid(M):

    M = M.astype(int)
    TMP = M.shape

    d = TMP[0]

    m_max = 0
    for i in range(0,d):
        if m_max < M[i]:
            m_max = M[i]

    X = np.zeros((m_max,d))
    W = np.zeros((m_max,d))

    for i in range(0,d):
        x, w = np.polynomial.legendre.leggauss(M[i])
        for j in range(0,M[i]):
            X[j,i] = x[j]
            W[j,i] = w[j]
    return X, W
