/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "gmm_extra.h"
#include <fstream>

using namespace tbox;

/**
 * @brief save CSR matrix to a file
 *
 *  matlab usage: load K.txt; K=spconvert(K)
 */

TBOX_API void
tomatlab(std::string const &fname, gmm::csr_matrix<double> const &A)
{
    std::ofstream f(fname.c_str());

    for (size_t i = 0; i < mat_nrows(A); ++i)
        for (size_t j = 0; j < mat_ncols(A); ++j)
        {
            double v = A(i, j);
            if (v != 0.0)
                f << i + 1 << ' ' << j + 1 << ' ' << v << '\n';
        }

    f.close();
}
