/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wCacheTetra4.h"

using namespace tbox;

CacheTetra4::CacheTetra4(int _npg) : Cache(_npg), gauss(_npg)
{
    // pre-compute dff
    dff.resize(gauss.p.size());
    for (size_t k = 0; k < gauss.p.size(); ++k)
    {
        Pt ksi(gauss.p[k]);
        dff[k].resize(SfTetra4::nbno);
        for (int i = 0; i < SfTetra4::nbno; ++i)
        {
            dff[k][i].resize(3);
            sf.evalD(i, ksi, dff[k][i]);
        }
    }

    // pre-compute ff
    ff.resize(gauss.p.size());
    for (size_t k = 0; k < gauss.p.size(); ++k)
    {
        Pt ksi(gauss.p[k]);
        ff[k].resize(SfTetra4::nbno);
        sf.eval(ksi, ff[k]);
    }
}

Gauss &CacheTetra4::getVGauss()
{
    return gauss;
}
