/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wSfLine2.h"
#include <stdexcept>

using namespace tbox;

void SfLine2::eval(Pt const &ksi, std::vector<double> &ff) const
{
    ff.resize(2);
    double x = ksi.x[0];

    ff[0] = 0.5 * (1.0 + x);
    ff[1] = 0.5 * (1.0 - x);
}

void SfLine2::evalD(int i, Pt const &ksi, std::vector<double> &dff) const
{
    dff.resize(1);
    //double x = ksi.x[0]; // unused

    switch (i)
    {
    case 0:
        dff[0] = 0.5;
        break;
    case 1:
        dff[0] = -0.5;
        break;
    default:
        throw std::runtime_error("bad index (SfLine2)!");
    }
}
