/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wSfTri3.h"
#include <stdexcept>

using namespace tbox;

//const int SfTri::nbno = 3;

void SfTri3::eval(Pt const &ksi, std::vector<double> &ff) const
{
    ff.resize(3);
    double x = ksi.x[0];
    double y = ksi.x[1];

    ff[0] = 1.0 - x - y;
    ff[1] = x;
    ff[2] = y;
}

void SfTri3::evalD(int i, Pt const &ksi, std::vector<double> &dff) const
{
    dff.resize(2);
    //double x = ksi.x[0];  // unused
    //double y = ksi.x[1];

    switch (i)
    {
    case 0:
        dff[0] = -1.0;
        dff[1] = -1.0;
        break;
    case 1:
        dff[0] = 1.0;
        dff[1] = 0.0;
        break;
    case 2:
        dff[0] = 0.0;
        dff[1] = 1.0;
        break;
    default:
        throw std::runtime_error("bad index (SfTri3)!");
    }
}
