/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WVTKEXPORT_H
#define WVTKEXPORT_H

#include "tboxVtk.h"
#include "wMshExport.h"
#include <vector>
#include <map>
#include <vtkSmartPointer.h>
#include <vtkUnstructuredGrid.h>

namespace tboxVtk
{

/**
 * @todo [Long-term] implement a way to automatically update vtk data structure (grid) if internal data structure (msh) changes.
 * For now, the points and the data are always updated at each save operation.
 */

/**
 * @brief Write mesh in VTK format
 * @authors Adrien Crovato, Romain Boman
 */
class TBOXVTK_API VtkExport : public tbox::MshExport
{
    vtkSmartPointer<vtkUnstructuredGrid> grid;
    void create();

public:
    VtkExport(std::shared_ptr<tbox::MshData> _msh);
    virtual ~VtkExport();

    virtual void save(std::string const &fname) const override;
#ifndef SWIG
    virtual void save(std::string const &fname, tbox::Results const &r) const override;
#endif

    virtual void write(std::ostream &out) const override;
};

} // namespace tboxVtk

#endif //WGMSHUTILS_H
