#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Simple 2D wave propagation test

from __future__ import print_function
from __future__ import division
from past.utils import old_div
import sys
#sys.path.append("/usr/local/bin")
#sys.path.append("C:\local\gmsh-2.9.0-Windows")

import wutils
wutils.findbins()
import waves as w
import tbox as tb

import wtools as wt

import onelab
olab = onelab.client(__file__)

time = olab.defineNumber('Dyna/time [s]', value=0.0)
wspeed = olab.defineNumber('Param/wave speed', value=10.0,
                            attributes={'Highlight':'Pink'})

if olab.action == 'check':
    exit(0)

def exportMshOpt():
    txt="""
//n = PostProcessing.NbViews - 1;
//If(n >= 0)
n=0;
Mesh.SurfaceEdges = 0;
Mesh.VolumeEdges = 0;
View[n].IntervalsType = 3;
View[n].CustomMax = 0.1;
View[n].CustomMin = 0;
View[n].NbIso = 16;
View[n].RangeType = 2;
View[n].SaturateValues = 1;
//EndIf
    """
    optFile = open(olab.getPath("results_merged.msh.opt"),'w')
    optFile.write(txt)
    optFile.close()

exportMshOpt()

class OneLabCallback(w.DisplayHook):
    def __init__(self, msh, scheme):
        w.DisplayHook.__init__(self)
        self.msh = msh
        self.scheme = scheme
        scheme.setGUI(self)
        scheme.wgmsh = w.Wgmsh_WG_ONE

    def display(self, nt, t, u):
        """ C++ => python callback
        """
        #print "[GUI] nt=", nt, "t=", t
        global olab

        ttot=self.scheme.ttot
        olab.setNumber(olab.name + '/Progress', value=t, min=0, max=ttot, visible=0)
        olab.setNumber('Dyna/time [s]', value=t)
        olab.setNumber('Dyna/step', value=nt)

        olab.setNumber('Solu/phi', value=u[0])
        olab.addNumberChoice('Solu/phi', u[0])



        # ask Gmsh to refresh
        olab.setString('Gmsh/Action', value='refresh')

        # stop if we are asked to (by Gmsh)
        if(olab.getString(olab.name + '/Action') == 'stop'):
            self.scheme.stopit=True

        filenames = ['waves01.msh', 'res.msh']
        with open('results_merged.msh', 'w') as outfile:
            for fname in filenames:
                with open(fname) as infile:
                    for line in infile:
                        outfile.write(line)
        olab.mergeFile(olab.checkPath('results_merged.msh'))
        #olab.mergeFile('results_merged.msh')

    def refresh(self):
        # donne la main au GUI a chaque pas
        pass
    def start(self):
        self.scheme.start()

def main():
    global olab
    msh = wutils.MeshLoader("waves01.geo").execute()
    print(msh)

    pbl = w.Problem(msh)
    print(pbl)

    Nx=50
    c = wspeed
    X=10.0;    Y=10.0;    Z=2.0
    freq=10.0
    lam = old_div(c, freq)
    lc = old_div(lam, 10)

    Le=old_div(X,Nx)
    dt=Le/c/10;

    scheme = w.ForwardEuler(msh, pbl)

    #wt.SinSource(msh, "Line2", 5.0)

    w.Medium(pbl, "Volume", c)

    if 0:
        wt.SinSource(pbl, "Line1", 10.0)
        w.Boundary(pbl, "FaceX1", c)
        w.Boundary(pbl, "FaceY1", c)

    if 0:
        wt.SinSource(pbl, "FaceX0", 10.0)
        w.Boundary(pbl, "FaceX1", c)

    if 1:
        s = wt.MultiSource(pbl, "FaceY0")
        s.addSource(tb.Pt(3,0,0),5)
        s.addSource(tb.Pt(7,0,0),20)
        w.Boundary(pbl, "FaceX0", c)
        w.Boundary(pbl, "FaceX1", c)
        w.Boundary(pbl, "FaceY1", c)

    if 0:
        wt.PulseSource(pbl, "Line1", 10.0, shift=0.3, width=100.0)
        w.Boundary(pbl, "FaceX1", c)
        w.Boundary(pbl, "FaceY1", c)

    # boundaries


    scheme.ttot = 10.0
    scheme.dt = dt
    scheme.savefreq=5
    scheme.nthreads=12


    usegui=1
    if not usegui:
        scheme.start()
    else:
        import viewer as v
        gui = OneLabCallback(msh, scheme)
        gui.start()


    #raw_input("<Press a key>")

if __name__ == "__main__":
    main()
